from TDhelper.Msg.InterfaceMsg import *
from TDhelper.apiCore import *

class SMS(Msg):
    '''手机短信
       
       Args:
       config (class<Config>): Open api config.
       sendBy (str): set system sendby,default value is "".
    '''
    def __init__(self, config:Config, sendBy:str=""):
        super(SMS,self).__init__(config, sendBy)
        
    def Send(self, sendTo:str='', sendBuff:str='', apiKey:str="send"):
        '''Send SMS method

            Args:
                sendTo (str): SMS recvice person, default value is None.
                sendBuff (str): SMS content, default value is "", will replace params (sendby,sendto,time.localtime())
                apiKey (str): send api keyword, default value is "send".
        '''
        if sendTo:
            argsTemplete=self._sendHandle.GetArgsString(apiKey)
            if argsTemplete:
                data= argsTemplete.format(str(sendTo),sendBuff.format(self._sendBy,sendTo,time.localtime()))  #create data
                return self._sendHandle.Call(apiKey, data)  #call remote api

    def MultipleSend(self, sendTo:array=[],sendBuff:str="", apiKey:str="multiplesend"):
        '''Multiple Send SMS method
        
            Args:
                sendTo (array): SMS recvice person, default value is [].
                sendBuff (str): SMS content, default value is "", will replace params (sendby,sendto,time.localtime())
                apiKey (str): send api keyword, default value is "multiplesend".
        '''
        if sendTo:
            argsTemplete=self._sendHandle.GetArgsString(apiKey)
            if argsTemplete:
                data= argsTemplete.format(str(sendTo),sendBuff.format(self._sendBy,sendTo,time.localtime()))  #create data
                return self._sendHandle.Call(apiKey, data)  #call remote api

    def Install(self):
        '''api config'''
        for item in self._config.Apis:
            self._sendHandle.AddApi(item.Key, item.Uri, item.argsTemplete, item.Type)
        self._sendHandle.SaveToFile()

    def Unstall(self):
        '''clear all api'''
        self._sendHandle.ClearCache()


import __init__
import unittest

from TDhelper.generic.requier import R
class TestReflect(unittest.TestCase):
    def test_reflect(self):
        bb=R("a.b:ab").Call('gg')
        print(bb)

if __name__ == "__main__":
    unittest.main()


import __init__
import unittest
from TDhelper.Scheduler.service import SchedulerService

class TestScheduler(unittest.TestCase):
    def test_scheduler(self):
        SCHEDULER_SERVICE= SchedulerService("//data")


if __name__ == "__main__":
    unittest.main()



import __init__
import unittest
from TDhelper.generic.recursion import recursion, recursionCall


@recursion
def permission(item, **kwargs):
    print(item, kwargs, "\r\n")
    if "children" in item:
        for o in item["children"]:
            recursionCall(permission, kwargs['limit'], o, **kwargs)
        kwargs["break"] = False
    else:
        kwargs["break"] = False
    return item, kwargs


class TestReflect(unittest.TestCase):
    def test_permission_call(self):
        _p = recursionCall(
            permission,
            200,{
                "permission_name": "APP Store",
                "permission_key": "APP_STORE",
                "permission_domain": "permission access domain",
                "permission_uri": "api/",
                "permission_enable": True,
                "permission_parent": 0,
                "children": [
                    {
                        "permission_name": "APP LIST",
                        "permission_key": "APP_STORE_APP_LISTS",
                        "permission_domain": "permission access domain",
                        "permission_uri": "api/apps",
                        "permission_enable": True,
                        "children": [
                            {
                                "permission_name": "APP CREATE",
                                "permission_key": "APP_STORE_APP_CREATE",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/apps",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "APP RETRIEVE",
                                "permission_key": "APP_STORE_APP_RETRIEVE",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/apps",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "APP UPDATE",
                                "permission_key": "APP_STORE_APP_UPDATE",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/apps",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "APP DESTROY",
                                "permission_key": "APP_STORE_APP_DESTROY",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/apps",
                                "permission_enable": True,
                            },
                        ],
                    },
                    {
                        "permission_name": "APP STORE ACCESS",
                        "permission_key": "APP_STORE_ACCESS",
                        "permission_domain": "permission access domain",
                        "permission_uri": "api/access",
                        "permission_enable": True,
                        "children": [
                            {
                                "permission_name": "APP STORE ACCESS LIST",
                                "permission_key": "APP_STORE_ACCESS_LIST",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/access",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "APP STORE ACCESS CREATE",
                                "permission_key": "APP_STORE_ACCESS_CREATE",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/access",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "APP STORE ACCESS RETRIEVE",
                                "permission_key": "APP_STORE_ACCESS_RETRIEVE",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/access",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "APP STORE ACCESS UPDATE",
                                "permission_key": "APP_STORE_ACCESS_UPDATE",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/access",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "APP STORE ACCESS DESTROY",
                                "permission_key": "APP_STORE_ACCESS_DESTROY",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/access",
                                "permission_enable": True,
                            },
                        ],
                    },
                    {
                        "permission_name": "DEVELOPER",
                        "permission_key": "DEVELOPER",
                        "permission_domain": " permission access domain",
                        "permission_uri": "api/developer",
                        "permission_enable": True,
                        "children": [
                            {
                                "permission_name": "DEVELOPER LIST",
                                "permission_key": "DEVELOPER_LIST",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/developer",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "DEVELOPER CREATE",
                                "permission_key": "DEVELOPER_CREATE",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/developer",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "DEVELOPER RETRIEVE",
                                "permission_key": "DEVELOPER_RETRIEVE",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/developer",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "DEVELOPER UPDATE",
                                "permission_key": "DEVELOPER_UPDATE",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/developer",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "DEVELOPER DESTROY",
                                "permission_key": "DEVELOPER_DESTROY",
                                "permission_domain": "permission access domain",
                                "permission_uri": "api/developer",
                                "permission_enable": True,
                            },
                        ],
                    },
                    {
                        "permission_name": "DEVELOPER TOKEN",
                        "permission_key": "DEVELOPER_TOKEN",
                        "permission_domain": "permission access domain",
                        "permission_uri": "api/token",
                        "permission_enable": True,
                        "children": [
                            {
                                "permission_name": "Developer token list",
                                "permission_key": "DEVELOPER_TOKEN_LIST",
                                "permission_domain": " permission access domain",
                                "permission_uri": "api/token",
                                "permission_enable": True,
                            },
                            {
                                "permission_name": "Developer token destroy",
                                "permission_key": "DEVELOPER_TOKEN_DESTROY",
                                "permission_domain": " permission access domain",
                                "permission_uri": "api/token",
                                "permission_enable": True,
                            },
                        ],
                    },
                ],
            },
            **{}
        )


if __name__ == "__main__":
    unittest.main()



import __init__
import unittest
import json
from TDhelper.network.http.RPC import RPC

class TestRPC(unittest.TestCase):
    def test_RPC_Register(self):
        pass

    def test_remote_call(self):
        RPC_CORE = RPC("http://192.168.50.2:9002/api/", "d74218cff240f446d341c4a2f3a8c588")
        RPC_LOADS=["APP_STORE"]
        class RPC_HANDLE:
            def __init__(self):
                self._cache = {}
                self.load()

            def call(self, serviceName, **args):
                assert (
                    len(serviceName.split(".")) == 2
                ), "serviceName formatter is errror. must (service.method)."
                m_service = serviceName.split(".")[0]
                if m_service in self._cache:
                    return self._cache[m_service].call(serviceName, **args)
                else:
                    return None

            def getHandle(self, key):
                if key in self._cache:
                    return self._cache[key]
                else:
                    return None

            def load(self):
                print("Begin load rpc service:")
                for m_prc_name in RPC_LOADS:
                    print("load %s" % m_prc_name)
                    m_rpc = RPC_CORE.handle(m_prc_name)
                    if m_rpc:
                        self._cache[m_prc_name] = m_rpc
                    else:
                        print("LOAD RPC %s error." % m_prc_name)
                print("End load rpc service.")

        # 在需要调用RPC的模块import R
        R = RPC_HANDLE()
        RPC_APP_STORE = R.getHandle("APP_STORE")
        ret=RPC_APP_STORE.call('APPVIEWSET_GETAPPSTATE',**{
            "developer_token":"*****",
            "app_token":"****"
        })
        print(ret)

if __name__ == "__main__":
    unittest.main()


import __init__
import unittest
from TDhelper.generic.recursion import recursion,recursionCall

@recursion
def fibonacci(i,step,**kwargs):
    i+=step if step else 1
    if i>=3000:
        kwargs["break"] = False
        return i,kwargs
    return fibonacci(i,step,**kwargs)

@recursion
def layers(item, **kwargs):
    print(item,kwargs,"\r\n")
    if "children" in item:
        for o in item["children"]:
            recursionCall(layers,kwargs["limit"],o, **kwargs)
        kwargs["break"] = False
    else:
        kwargs["break"]= False
    return item, kwargs

class TestReflect(unittest.TestCase):

    def test_recursion_fibonacci_call(self):
        b=recursionCall(fibonacci,100,1,1,**{})
        print("end",b,"\r\n")

    def test_recursion_more_layer_call(self):
        b = recursionCall(
                layers,
                2,
                {
                    "name": "a",
                    "children": [
                        {
                            "name": "b",
                            "children": [
                                {"name": "c", "children": [{"name": "d"}]},
                                {"name": "e", "children": [{"name": "f"}]},
                            ],
                        },
                        {
                            "name": "g",
                            "children": [
                                {"name": "h", "children": [{"name": "i"}]},
                                {"name": "j", "children": [{"name": "k","children":[
                                    {"name": "l","children":[
                                        {"name": "m","children":[
                                            {"name": "n","children":[
                                                {"name": "o","children":[
                                                    {"name": "p","children":[
                                                        {"name": "q","children":[
                                                            {"name": "r","children":[
                                                                {"name": "s","children":[
                                                                    {"name": "t","children":[
                                                                        {"name": "u","children":[
                                                                            {"name": "v","children":[
                                                                                {"name": "w","children":[
                                                                                    {"name": "x","children":[
                                                                                        {"name": "y","children":[
                                                                                            {"name": "z"}
                                                                                        ]}
                                                                                    ]}
                                                                                ]}
                                                                            ]}
                                                                        ]}
                                                                    ]}
                                                                ]}
                                                            ]}
                                                        ]}
                                                    ]}
                                                ]}
                                            ]}
                                        ]}
                                    ]}
                                ]}]},
                            ],
                        },
                    ],
                },
                **{}
            )

if __name__ == "__main__":
    unittest.main()


import json
import copy
import random
from typing import List, Tuple, Dict
from urllib import parse
from types import FunctionType
from TDhelper.generic.classDocCfg import doc
from network.http.REST_HTTP import GET, POST, PUT, DELETE, ContentType
from TDhelper.Decorators.log import logging, logging_setup
from TDhelper.generic.randGetValue import getValue

## 以后整理。测试用
class RPCRegister:
    def __init__(self, serviceConfig, HostConfig, rpcConfig):
        self._serviceConfig = serviceConfig
        self._hostConfig = HostConfig
        self._access_token = rpcConfig["token"]
        self._m_heads = {}
        self._m_heads["api-token"] = self._access_token if self._access_token else ""
        # rpc uri传数组过来，从数组里随机取一个服务地址。后续需要做负载均衡。
        self._sc_uri = getValue(rpcConfig["uri"]).strip("/") + "/"

    def RegisterRPC(self):
        return self._registerService()

    def RegisterMethod(self, pk, serviceClass):
        self._registerMehotd(pk, serviceClass)

    def _registerService(self):
        m_service_post_data = ""
        m_count = 0
        # 生成注册服务参数.
        for k, v in self._serviceConfig.items():
            if k.lower() != "description":
                if not v:
                    raise Exception("service '%s' value can't is none." % k)
            m_service_post_data += k + "=" + parse.quote(str(v))
            m_count += 1
            if m_count < len(self._serviceConfig):
                m_service_post_data += "&"
        # 注册服务基本信息.
        state, ret = POST(
            self._sc_uri + "services/".strip("/") + "/",
            post_data=bytes(m_service_post_data, encoding="utf-8"),
            http_headers=self._m_heads,
            time_out=15,
        )
        if state == 200:
            ret = str(ret, encoding="utf-8")
            ret = json.loads(ret, encoding="utf-8")
            if ret["state"] == 200:
                logging.info(
                    "%s(%s) register success."
                    % (self._serviceConfig["key"], self._serviceConfig["name"])
                )
                if ret["msg"]["id"]:
                    self._registerHost(ret["msg"]["id"], self._hostConfig)
                    # self._registerMehotd(ret["msg"]["id"], serviceClass)
                    return ret["msg"]["id"]
                else:
                    return None
            else:
                return None
        else:
            return None

    def _registerHost(self, pk, hosts):
        # 注册服务器信息.
        for i in range(0, len(hosts)):
            hosts[i]["service"] = pk
            hosts[i]["state"] = True
            m_count = 0
            m_service_hosts_post_data = ""
            for k, v in hosts[i].items():
                if v:
                    m_service_hosts_post_data += k + "=" + parse.quote(str(v))
                    m_count += 1
                    if m_count < len(hosts[i]):
                        m_service_hosts_post_data += "&"
                else:
                    raise Exception("register hosts {} can't is none." % k)
            state, ret = POST(
                self._sc_uri + "hosts/".strip("/") + "/",
                post_data=bytes(m_service_hosts_post_data, encoding="utf-8"),
                http_headers=self._m_heads,
                time_out=15,
            )
            if state != 200:
                logging.error(
                    "register service hosts failed. msg:{}" % str(ret, encoding="utf-8")
                )
            else:
                ret = json.loads(str(ret, encoding="utf-8"))
                if ret["state"] != 200:
                    m_msg = "register service hosts failed. http code({}), msg:{}" % (
                        ret["state"],
                        ret["msg"],
                    )
                    logging.error(m_msg)
                else:
                    logging.info(
                        "register service hosts(%s:%s) success."
                        % (hosts[i]["host"], hosts[i]["port"])
                    )

    def _registerMehotd(self, pk, serviceClass):
        # 注册方法
        for k, v in serviceClass.__dict__.items():
            if isinstance(v, FunctionType):
                func_name = v.__name__.upper()
                if v.__doc__:
                    methods = doc(v.__doc__, "rpc")
                    if methods:
                        methods = methods.replace("\n", "").strip()
                        try:
                            methods = [json.loads(methods, encoding="utf-8")]
                            for i in range(0, len(methods)):
                                # todo register method.
                                methods[i]["service"] = pk
                                if not "key" in methods[i]:
                                    methods[i]["key"] = (
                                        self._serviceConfig["key"].upper()
                                        + "."
                                        + serviceClass.__name__.upper()
                                        + "_"
                                        + func_name
                                    )
                                else:
                                    if not methods[i]["key"]:
                                        methods[i]["key"] = (
                                            self._serviceConfig["key"].upper()
                                            + "."
                                            + serviceClass.__name__.upper()
                                            + "_"
                                            + func_name
                                        )
                                    else:
                                        if len(methods[i]["key"].split(".")) == 1:
                                            methods[i]["key"] = (
                                                self._serviceConfig["key"].upper()
                                                + "."
                                                + serviceClass.__name__.upper()
                                                + "_"
                                                + methods[i]["key"]
                                            )
                                        else:
                                            logging.error(
                                                "%s key can not contain (.) ."
                                                % methods[i]["key"]
                                            )
                                m_service_uri_post_data = ""
                                if not methods[i]["uri"].endswith("/"):
                                    methods[i]["uri"] += "/"
                                m_count = 0
                                for k, v in methods[i].items():
                                    if k.lower() != "params":
                                        if (
                                            k.lower() == "key"
                                            or k.lower() == "uri"
                                            or k.lower() == "method"
                                        ):
                                            if not v:
                                                raise Exception(
                                                    "methods (%s) is none." % k.lower()
                                                )
                                        if k.lower() != "method":
                                            m_service_uri_post_data += (
                                                k + "=" + parse.quote(str(v))
                                            )
                                        else:
                                            m_value = 0
                                            if v.upper() == u"GET":
                                                m_value = 1
                                            elif v.upper() == u"POST":
                                                m_value = 2
                                            elif v.upper() == u"PUT":
                                                m_value = 3
                                            elif v.upper() == u"DELETE":
                                                m_value = 4
                                            m_service_uri_post_data += (
                                                k + "=" + str(m_value)
                                            )
                                        m_count += 1
                                        if "params" in methods[i]:
                                            if m_count < len(methods[i]) - 1:
                                                m_service_uri_post_data += "&"
                                        else:
                                            if m_count < len(methods[i]):
                                                m_service_uri_post_data += "&"
                                state, ret = POST(
                                    self._sc_uri + "uri/".strip("/") + "/",
                                    post_data=bytes(
                                        m_service_uri_post_data, encoding="utf8"
                                    ),
                                    http_headers=self._m_heads,
                                    time_out=15,
                                )
                                if state != 200:
                                    logging.error(
                                        "register service methods failed. msg:{}"
                                        % str(ret, encoding="utf-8")
                                    )
                                else:
                                    ret = json.loads(str(ret, encoding="utf-8"))
                                    if ret["state"] != 200:
                                        m_msg = (
                                            "register service methods failed. http code(%d), msg:%s"
                                            % (ret["state"], ret["msg"])
                                        )
                                        logging.error(m_msg)
                                    else:
                                        logging.info(
                                            "%s register success." % methods[i]["key"]
                                        )
                                        if ret:
                                            if "params" in methods[i]:
                                                params = methods[i]["params"]
                                                m_method_id = ret["msg"]["id"]
                                                if params:
                                                    # 有参数能注册.
                                                    for param_offset in range(
                                                        0, len(params)
                                                    ):
                                                        params[param_offset][
                                                            "serviceUri"
                                                        ] = m_method_id
                                                        m_service_method_params_post_data = (
                                                            ""
                                                        )
                                                        m_count = 0
                                                        for k, v in params[
                                                            param_offset
                                                        ].items():
                                                            if not v:
                                                                raise Exception(
                                                                    "methods params (%s) is none."
                                                                    % k.lower()
                                                                )
                                                            m_service_method_params_post_data += (
                                                                k
                                                                + "="
                                                                + parse.quote(str(v))
                                                            )
                                                            m_count += 1
                                                            try:
                                                                if m_count < len(
                                                                    params[param_offset]
                                                                ):
                                                                    m_service_method_params_post_data += (
                                                                        "&"
                                                                    )
                                                            except Exception as e:
                                                                raise e
                                                        state, ret = POST(
                                                            self._sc_uri
                                                            + "params/".strip("/")
                                                            + "/",
                                                            post_data=bytes(
                                                                m_service_method_params_post_data,
                                                                encoding="utf-8",
                                                            ),
                                                            http_headers=self._m_heads,
                                                            time_out=15,
                                                        )
                                            # 注册返回值
                                            if "returns" in methods[i]:
                                                m_returns = methods[i]["returns"]
                                                if m_returns:
                                                    # 有返回值设置进行注册.
                                                    reg_returns_params = ""
                                                    m_count = 0
                                                    m_returns[
                                                        "serviceUri"
                                                    ] = m_method_id
                                                    for k, v in m_returns.items():
                                                        if k.lower() != "descriptions":
                                                            reg_returns_params += (
                                                                k
                                                                + "="
                                                                + parse.quote(str(v))
                                                            )
                                                            m_count += 1
                                                            if (
                                                                "descriptions"
                                                                in m_returns
                                                            ):
                                                                if (
                                                                    m_count
                                                                    < len(m_returns) - 1
                                                                ):
                                                                    reg_returns_params += (
                                                                        "&"
                                                                    )
                                                            else:
                                                                if m_count < len(
                                                                    m_returns
                                                                ):
                                                                    reg_returns_params += (
                                                                        "&"
                                                                    )
                                                    state, ret = POST(
                                                        self._sc_uri
                                                        + "returns/".strip("/")
                                                        + "/",
                                                        post_data=bytes(
                                                            reg_returns_params,
                                                            encoding="utf-8",
                                                        ),
                                                        http_headers=self._m_heads,
                                                        time_out=15,
                                                    )
                                                    if state == 200:
                                                        ret = json.loads(
                                                            str(ret, encoding="utf-8")
                                                        )
                                                        if ret["state"] != 200:
                                                            m_msg = "register service methods failed. http code(%d), msg:%s" % (
                                                                ret["state"],
                                                                ret["msg"],
                                                            )
                                                            logging.error(m_msg)
                                                        else:
                                                            m_returns_id = ret["msg"][
                                                                "id"
                                                            ]
                                                            if (
                                                                "descriptions"
                                                                in m_returns
                                                            ):
                                                                # 注册返回值说明
                                                                for item in m_returns[
                                                                    "descriptions"
                                                                ]:
                                                                    m_returns_description = (
                                                                        "returns="
                                                                        + str(
                                                                            m_returns_id
                                                                        )
                                                                    )
                                                                    m_returns_description += (
                                                                        "&key="
                                                                        + parse.quote(
                                                                            item["key"]
                                                                        )
                                                                    )
                                                                    m_description = item[
                                                                        "valueDescription"
                                                                    ]
                                                                    m_description = m_description.replace(
                                                                        "<", "&lt;"
                                                                    )
                                                                    m_description = m_description.replace(
                                                                        ">", "&gt;"
                                                                    )
                                                                    m_description = m_description.replace(
                                                                        "\r\n", "<br />"
                                                                    )
                                                                    m_returns_description += (
                                                                        "&valueDescription="
                                                                        + parse.quote(
                                                                            m_description
                                                                        )
                                                                    )
                                                                    state, ret = POST(
                                                                        self._sc_uri
                                                                        + "returnDescriptons/".strip(
                                                                            "/"
                                                                        )
                                                                        + "/",
                                                                        post_data=bytes(
                                                                            m_returns_description,
                                                                            encoding="utf-8",
                                                                        ),
                                                                        http_headers=self._m_heads,
                                                                        time_out=15,
                                                                    )
                                                                    if state != 200:
                                                                        m_msg = (
                                                                            "register return description failed. http code(%d)"
                                                                            % state
                                                                        )
                                                                        logging.error(
                                                                            m_msg
                                                                        )
                                                                    else:
                                                                        ret = json.loads(
                                                                            str(
                                                                                ret,
                                                                                encoding="utf-8",
                                                                            )
                                                                        )
                                                                        if (
                                                                            ret["state"]
                                                                            != 200
                                                                        ):
                                                                            m_msg = "register return description failed. state(%d), msg(%s), key()" % (
                                                                                ret[
                                                                                    "state"
                                                                                ],
                                                                                ret[
                                                                                    "msg"
                                                                                ],
                                                                                item[
                                                                                    "key"
                                                                                ],
                                                                            )
                                                                            logging.error(
                                                                                m_msg
                                                                            )
                                                    else:
                                                        m_msg = (
                                                            "register returns failed. http code(%d)"
                                                            % state
                                                        )
                                                        logging.error(m_msg)

                                        else:
                                            if ret["state"] != 200:
                                                m_msg = (
                                                    "register service hosts failed. http code(%d), msg:%s"
                                                    % (ret["state"], ret["msg"])
                                                )
                                                logging.error(m_msg)
                        except Exception as e:
                            logging.error(e)
                            logging.error(
                                "register params error. method config error.please checked %s.__doc__"
                                % (v.__name__)
                            )


class RPC:
    """
    RPC. 此类只能配合webservice/rpc使用, 独立使用将会报错.
    """

    def __init__(self, service_center_uri, secret):
        """
        初始化
        - params:
        -   service_center_uri:<string>, 服务中心获取API接口URI.
        -   secret: <string>, 访问密钥
        """
        self._access_token = secret
        self._m_heads = {}
        self._m_heads["api-token"] = self._access_token if self._access_token else ""
        self._sc_uri = getValue(service_center_uri).rstrip("/") + "/"
        self._apisTable = {}
        self._current_service = None

    def RPC_ACCESS(
        self,
        entryPoint: List[str],
        access_token: str,
        method: str,
        **kwargs: "dict|Tuple"
    ):
        method = parse.quote(method)
        params = ""
        headers = {}
        if "headers" in kwargs:
            headers = kwargs["headers"]
        headers["api-token"] = access_token
        for o in kwargs:
            if o.lower() != "headers":
                params += o + "=" + kwargs[o]
        params = "?" + params
        result = {"state": False, "msg": ""}
        for uri in entryPoint:
            uri = uri.rstrip("/") + "/" + method.strip("/") + "/" + params
            status, body = GET(uri=uri, time_out=15, http_headers=headers)
            if status == 200:
                body = json.loads(str(body, encoding="utf-8"))
                if body["state"] == 200:
                    result["state"] = True
                    result["msg"] = body["msg"]
                break
        return result["state"], result["msg"]

    def _getApi(self, service, method):
        if service + method not in self._apisTable:
            # 远程获取
            method = parse.quote(method)
            status, body = GET(
                uri=self._sc_uri + "rpc/?key=" + service + "&method=" + method,
                time_out=15,
                http_headers=self._m_heads,
            )
            if status == 200:
                self._apisTable[service + method] = json.loads(
                    str(body, encoding="utf-8")
                )
        # 从本地获取API配置
        if service + method in self._apisTable:
            if self._apisTable[service + method]["state"] == 200:
                return True, self._apisTable[service + method]["msg"]
            else:
                return False, self._apisTable[service + method]["msg"]
        else:
            return False, "can't found %s api." % service + method

    def register(self, service: Dict, hosts: List, methods: List) -> Tuple:
        """
        注册服务
        - params:
        -   service: <dict>, 服务信息. formatter:{"name":"","description":"","key":"","httpProtocol":""}
        -   hosts: <[]>, 服务器信息. formatter:[{"host":"ip地址","port":端口}]
        -   methods: <[]>, 方法. formatter: [{"key":"方法索引","uri":"api url","method":"GET|POST|PUT|DELETE","version":"版本号","description":"描述", "params":[{"key":"参数名称","description":"描述","defaultValue":"默认值(调用不传参时默认值)"}],"returns":[{"valueType":"json|xml","examples":"html code","descriptions":[{"key":"值key","valueDescription":"值描述"}]}]}]
        - Returns:
        -   bool, str: 状态，信息.
        """
        m_service_post_data = ""
        m_count = 0
        # 生成注册服务参数.
        for k, v in service.items():
            if k.lower() != "description":
                if not v:
                    raise Exception("service '%s' value can't is none." % k)
            m_service_post_data += k + "=" + parse.quote(str(v))
            m_count += 1
            if m_count < len(service):
                m_service_post_data += "&"
        # 注册服务基本信息.
        state, ret = POST(
            self._sc_uri + "services/",
            post_data=bytes(m_service_post_data, encoding="utf-8"),
            http_headers=self._m_heads,
            time_out=15,
        )
        if state == 200:
            if ret:
                ret = json.loads(str(ret, encoding="utf-8"))
                if ret["state"] == 200:
                    m_service_id = ret["msg"]["id"]
                    # 注册服务器信息.
                    for i in range(0, len(hosts)):
                        hosts[i]["service"] = m_service_id
                        # hosts[i]['state']= True
                        m_count = 0
                        m_service_hosts_post_data = ""
                        for k, v in hosts[i].items():
                            if v:
                                m_service_hosts_post_data += (
                                    k + "=" + parse.quote(str(v))
                                )
                                m_count += 1
                                if m_count < len(hosts[i]):
                                    m_service_hosts_post_data += "&"
                            else:
                                raise Exception("register hosts {} can't is none." % k)
                        state, ret = POST(
                            self._sc_uri + "hosts/",
                            post_data=bytes(
                                m_service_hosts_post_data, encoding="utf-8"
                            ),
                            http_headers=self._m_heads,
                            time_out=15,
                        )
                        if state != 200:
                            return False, "register service hosts failed. msg:{}" % str(
                                ret, encoding="utf-8"
                            )
                        else:
                            ret = json.loads(str(ret, encoding="utf-8"))
                            if ret["state"] != 200:
                                m_msg = (
                                    "register service hosts failed. http code({}), msg:{}"
                                    % (ret["state"], ret["msg"])
                                )
                                return False, m_msg
                    # 注册方法
                    for i in range(0, len(methods)):
                        # todo register method.
                        methods[i]["service"] = m_service_id
                        m_service_uri_post_data = ""
                        m_count = 0
                        for k, v in methods[i].items():
                            if k.lower() != "params":
                                if (
                                    k.lower() == "key"
                                    or k.lower() == "uri"
                                    or k.lower() == "method"
                                ):
                                    if not v:
                                        raise Exception(
                                            "methods (%s) is none." % k.lower()
                                        )
                                if k.lower() != "method":
                                    m_service_uri_post_data += (
                                        k + "=" + parse.quote(str(v))
                                    )
                                else:
                                    m_value = 0
                                    if v.upper() == u"GET":
                                        m_value = 1
                                    elif v.upper() == u"POST":
                                        m_value = 2
                                    elif v.upper() == u"PUT":
                                        m_value = 3
                                    elif v.upper() == u"DELETE":
                                        m_value = 4
                                    m_service_uri_post_data += k + "=" + str(m_value)
                                m_count += 1
                                if "params" in methods[i]:
                                    if m_count < len(methods[i]) - 1:
                                        m_service_uri_post_data += "&"
                                else:
                                    if m_count < len(methods[i]):
                                        m_service_uri_post_data += "&"
                        state, ret = POST(
                            self._sc_uri + "uri/",
                            post_data=bytes(m_service_uri_post_data, encoding="utf8"),
                            http_headers=self._m_heads,
                            time_out=15,
                        )
                        if state != 200:
                            return (
                                False,
                                "register service methods failed. msg:{}"
                                % str(ret, encoding="utf-8"),
                            )
                        else:
                            ret = json.loads(str(ret, encoding="utf-8"))
                            if ret["state"] != 200:
                                m_msg = (
                                    "register service methods failed. http code(%d), msg:%s"
                                    % (ret["state"], ret["msg"])
                                )
                                return False, m_msg
                            else:
                                if ret:
                                    if "params" in methods[i]:
                                        params = methods[i]["params"]
                                        m_method_id = ret["msg"]["id"]
                                        if params:
                                            # 有参数能注册.
                                            for param_offset in range(0, len(params)):
                                                params[param_offset][
                                                    "serviceUri"
                                                ] = m_method_id
                                                m_service_method_params_post_data = ""
                                                m_count = 0
                                                for k, v in params[
                                                    param_offset
                                                ].items():
                                                    if not v:
                                                        raise Exception(
                                                            "methods params (%s) is none."
                                                            % k.lower()
                                                        )
                                                    m_service_method_params_post_data += (
                                                        k + "=" + parse.quote(str(v))
                                                    )
                                                    m_count += 1
                                                    try:
                                                        if m_count < len(
                                                            params[param_offset]
                                                        ):
                                                            m_service_method_params_post_data += (
                                                                "&"
                                                            )
                                                    except Exception as e:
                                                        raise e
                                                state, ret = POST(
                                                    self._sc_uri + "params/",
                                                    post_data=bytes(
                                                        m_service_method_params_post_data,
                                                        encoding="utf-8",
                                                    ),
                                                    http_headers=self._m_heads,
                                                    time_out=15,
                                                )
                                    # 注册返回值
                                    if "returns" in methods[i]:
                                        m_returns = methods[i]["returns"]
                                        if m_returns:
                                            # 有返回值设置进行注册.
                                            reg_returns_params = ""
                                            m_count = 0
                                            m_returns["serviceUri"] = m_method_id
                                            for k, v in m_returns.items():
                                                if k.lower() != "descriptions":
                                                    reg_returns_params += (
                                                        k + "=" + parse.quote(str(v))
                                                    )
                                                    m_count += 1
                                                    if "descriptions" in m_returns:
                                                        if m_count < len(m_returns) - 1:
                                                            reg_returns_params += "&"
                                                    else:
                                                        if m_count < len(m_returns):
                                                            reg_returns_params += "&"
                                            state, ret = POST(
                                                self._sc_uri + "returns/",
                                                post_data=bytes(
                                                    reg_returns_params, encoding="utf-8"
                                                ),
                                                http_headers=self._m_heads,
                                                time_out=15,
                                            )
                                            if state == 200:
                                                ret = json.loads(
                                                    str(ret, encoding="utf-8")
                                                )
                                                if ret["state"] != 200:
                                                    m_msg = (
                                                        "register service methods failed. http code(%d), msg:%s"
                                                        % (ret["state"], ret["msg"])
                                                    )
                                                    return False, m_msg
                                                else:
                                                    m_returns_id = ret["msg"]["id"]
                                                    if "descriptions" in m_returns:
                                                        # 注册返回值说明
                                                        for item in m_returns[
                                                            "descriptions"
                                                        ]:
                                                            m_returns_description = (
                                                                "returns="
                                                                + str(m_returns_id)
                                                            )
                                                            m_returns_description += (
                                                                "&key="
                                                                + parse.quote(
                                                                    item["key"]
                                                                )
                                                            )
                                                            m_description = item[
                                                                "valueDescription"
                                                            ]
                                                            m_description = (
                                                                m_description.replace(
                                                                    "<", "&lt;"
                                                                )
                                                            )
                                                            m_description = (
                                                                m_description.replace(
                                                                    ">", "&gt;"
                                                                )
                                                            )
                                                            m_description = (
                                                                m_description.replace(
                                                                    "\r\n", "<br />"
                                                                )
                                                            )
                                                            m_returns_description += (
                                                                "&valueDescription="
                                                                + parse.quote(
                                                                    m_description
                                                                )
                                                            )
                                                            state, ret = POST(
                                                                self._sc_uri
                                                                + "returnDescriptons/",
                                                                post_data=bytes(
                                                                    m_returns_description,
                                                                    encoding="utf-8",
                                                                ),
                                                                http_headers=self._m_heads,
                                                                time_out=15,
                                                            )
                                                            if state != 200:
                                                                m_msg = (
                                                                    "register return description failed. http code(%d)"
                                                                    % state
                                                                )
                                                                return False, m_msg
                                                            else:
                                                                ret = json.loads(
                                                                    str(
                                                                        ret,
                                                                        encoding="utf-8",
                                                                    )
                                                                )
                                                                if ret["state"] != 200:
                                                                    m_msg = "register return description failed. state(%d), msg(%s), key()" % (
                                                                        ret["state"],
                                                                        ret["msg"],
                                                                        item["key"],
                                                                    )
                                                                    return False, m_msg
                                            else:
                                                m_msg = (
                                                    "register returns failed. http code(%d)"
                                                    % state
                                                )
                                                return False, m_msg

                                else:
                                    if ret["state"] != 200:
                                        m_msg = (
                                            "register service hosts failed. http code(%d), msg:%s"
                                            % (ret["state"], ret["msg"])
                                        )
                                        return False, m_msg
                    return True, "register service success."
                else:
                    msg = "register service http error. http code(%s), msg:%s" % (
                        ret["state"],
                        ret["msg"],
                    )
                    return False, msg
            else:
                raise Exception("register return body is none, please checked.")
        else:
            msg = "register service http error. http code(%d), msg:%s" % (
                state,
                str(ret, encoding="utf-8"),
            )
            return False, msg

    def handle(self, service):
        """
        获取服务句柄
        - params:
        -   service: <string>, 服务key
        - returns
        -   self: <RPC>, 返回一个RPC服务句柄.
        """
        self._current_service = service
        return copy.deepcopy(self)

    def call(self, method, **kwargs):
        """
        调用方法
        - params:
        -   method: <string>, 方法key值.
        - **kwargs: 参数字典
        - returns:
        -   json: <json>, formatter: {"state":"调用网络状态", "msg":"调用的方法的返回值"}
        """
        if not self._current_service:
            raise Exception(
                "service is not set, first call self.handle method to set service."
            )
        state, ret = self._getApi(
            self._current_service, self._current_service + "." + method
        )
        if state:
            try:
                m_uri = ret["uri"]
                m_params = ret["params"]
                m_method = ret["method"]
                m_headers = {}
                if "headers" in kwargs:
                    if isinstance(kwargs["headers"], dict):
                        for o in kwargs["headers"]:
                            m_headers[
                                o.lower().replace("http_", "").replace("_", "-")
                            ] = kwargs["headers"][o]
                    else:
                        return {
                            "state": -1,
                            "msg": "http request headers must is dict type.",
                        }
                m_headers["api-token"] = ret["secret"] if ret["secret"] else ""
                params_data = ""
                m_count = 0
                for param in m_params:
                    if param["key"].lower() != "pk":
                        if param["key"].startswith("http_header_") or param[
                            "key"
                        ].startswith("HTTP_HEADER_"):
                            if param["key"] not in m_headers:
                                if param["key"] not in kwargs:
                                    return {
                                        "state": -1,
                                        "msg": "not found parameter '%s'"
                                        % (param["key"]),
                                    }
                                m_headers[
                                    param["key"]
                                    .replace("_", "-")
                                    .replace("http-header-", "")
                                    .replace("HTTP-HEADER-", "")
                                ] = kwargs[param["key"]]
                        else:
                            if param["key"] in kwargs:
                                params_data += (
                                    param["key"]
                                    + "="
                                    + (
                                        parse.quote(kwargs[param["key"]])
                                        if isinstance(kwargs[param["key"]], str)
                                        else parse.quote(str(kwargs[param["key"]]))
                                    )
                                )
                            else:
                                if param["defaultValue"]:
                                    params_data += (
                                        param["key"]
                                        + "="
                                        + (
                                            parse.quote(param["defaultValue"])
                                            if isinstance(param["defaultValue"], str)
                                            else parse.quote(str(param["defaultValue"]))
                                        )
                                    )
                            m_count += 1
                            if m_count < len(m_params):
                                if len(params_data) > 0:
                                    params_data += "&"
                    else:
                        m_uri = m_uri.replace(
                            "{pk}",
                            parse.quote(kwargs[param["key"]])
                            if isinstance(kwargs[param["key"]], str)
                            else parse.quote(str(kwargs[param["key"]])),
                        )
                if params_data:
                    if m_method == u"GET" or m_method == u"DELETE":
                        if m_uri.find("?") >= 0:
                            m_uri += "&" + params_data
                        else:
                            m_uri += "?" + params_data
            except Exception as e:
                return {"state": -1, "msg": e.args}
            try:
                if m_method == u"GET":
                    logging.info("access api:(%s), method(GET)." % m_uri)
                    state, ret = GET(uri=m_uri, http_headers=m_headers, time_out=15)
                    if state == 200:
                        try:
                            ret = json.loads(str(ret, encoding="utf-8"))
                            return {"state": ret["state"], "msg": ret["msg"]}
                        except Exception as e:
                            logging.error(e.args)
                            return {
                                "state": state,
                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
                            }
                    else:
                        try:
                            if ret:
                                ret = json.loads(str(ret, encoding="utf-8"))
                                return {"state": ret["state"], "msg": ret["msg"]}
                            else:
                                return {"state": state, "msg": "%s" % m_uri}
                        except Exception as e:
                            return {
                                "state": state,
                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
                            }
                elif m_method == u"POST":
                    logging.info(
                        "access api:(%s), method(POST),postdata:%s."
                        % (m_uri, kwargs["data"])
                    )
                    state, ret = POST(
                        m_uri,
                        kwargs["data"],
                        content_type=ContentType.JSON,
                        http_headers=m_headers,
                        time_out=15,
                    )
                    if state == 200:
                        try:
                            ret = json.loads(str(ret, encoding="utf-8"))
                            return {"state": ret["state"], "msg": ret["msg"]}
                        except Exception as e:
                            logging.error(e.args)
                            return {
                                "state": state,
                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
                            }
                    else:
                        try:
                            if ret:
                                ret = json.loads(str(ret, encoding="utf-8"))
                                return {"state": ret["state"], "msg": ret["msg"]}
                            else:
                                return {"state": state, "msg": "%s" % m_uri}
                        except Exception as e:
                            logging.error(e.args)
                            return {
                                "state": state,
                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
                            }
                elif m_method == u"PUT":
                    logging.info(
                        "access api:(%s), method(PUT),postdata:%s."
                        % (m_uri, params_data)
                    )

                    # 还没有写PUT方法
                    raise Exception("urllib PUT方法还没写.")
                elif m_method == u"DELETE":
                    # 还没有写DELETE方法
                    logging.info("access api:(%s), method(DELETE)" % m_uri)
                    if "data" in kwargs:
                        state, ret = DELETE(
                            m_uri,
                            post_data=kwargs["data"],
                            http_headers=m_headers,
                            content_type=ContentType.JSON,
                            time_out=15,
                        )
                    else:
                        state, ret = DELETE(
                            uri=m_uri, http_headers=m_headers, time_out=15
                        )
                    if state==200:
                        try:
                            ret= json.loads(str(ret,encoding='utf-8'))
                            return {"state":ret["state"], "msg": ret["msg"]}
                        except Exception as e:
                            logging.error(e.args)
                            return {
                                "state": state,
                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
                            }
                    else:
                        try:
                            if ret:
                                ret = json.loads(str(ret, encoding="utf-8"))
                                return {"state": ret["state"], "msg": ret["msg"]}
                            else:
                                return {"state": state, "msg": "%s" % m_uri}
                        except Exception as e:
                            logging.error(e.args)
                            return {
                                "state": state,
                                "msg": 'remote call "%s" error.(%s)' % (m_uri,e.args),
                            }
                else:
                    return {"state":-1, "msg":"restful method(%s) is error. POST|GET|DELETE|PUT(unrealized)|OPTIONS(unrealized)|HEAD(unrealized)"%(m_method)}
            except Exception as e:
                return {"state": -1, "msg": e.args}
        else:
            return {"state": -1, "msg": ret}



#!/usr/bin/env python3.6
# -*- encoding: utf-8 -*-
'''
@File    :   dictHelper.py
@Time    :   2020/09/24 07:13:30
@Author  :   Tang Jing
@Version :   1.0.0
@Contact :   yeihizhi@163.com
@License :   (C)Copyright 2020
@Desc    :   None
'''

# here put the import lib

# code start


def createDictbyStr(key: str, source={}, value: any = None):
    '''
        create an dict object by formatter string.
        - Params:
        -   key: <str>, the string that generates the dict.
        -   source: <dict>, the targte dict, default {}.
        -   value: <any>, set the value of the new dict last child node.
        - Returns: <dict>, the new dict.
    '''
    result = source
    node = ''
    total = 1
    m_source = result
    for k in key.split('.'):
        total += 1
        node += ("." if node else "") + k
        if node:
            search_count = 1
            m_m_source = m_source
            for k1 in node.split('.'):
                search_count += 1
                if search_count > len(node.split('.')):
                    if k1 not in m_m_source:
                        m_m_source.update({k1: value})
                    else:
                        m_m_source = m_m_source[k1]
                else:
                    if k1 not in m_m_source:
                        m_m_source.update({k1: {}})
                    else:
                        if not isinstance(m_m_source[k1], dict):
                            m_m_source[k1] = {}
                    m_m_source = m_m_source[k1]
    return result


def findInDict(findKey, source: dict):
    '''
        根据finkey查找dict.
        - Params:
        -   findkey: <str>, formatter("node1.node2.node3").
        -   source: <dict>, it is your search target, findkey must in it. if findkey not in it, then raise an error
        - Returns: <any>, if can't found, then return None.
    '''
    result = None
    search_node = ''
    for key in findKey.split('.'):
        search_node += ("." if search_node else "") + key
        if key not in source:
            raise Exception("can not found key(%s)" % search_node)
        result = source[key]
        if isinstance(source[key], dict):
            source = source[key]
    return result



import __init__
import unittest
import datetime
from TDhelper.document.excel.model import model
from TDhelper.document.excel.FieldType import FieldType

class stock(model):
    stock_date = FieldType(datetime.date, 1)
    stock_code = FieldType(str, 2)
    stock_name = FieldType(str, 3)
    stock_close_price = FieldType(float, 4)
    stock_max_price = FieldType(float, 5)
    stock_min_price = FieldType(float, 6)
    stock_open_price = FieldType(float, 7)
    stock_yesterday_close_price = FieldType(float, 8)
    stock_rise_and_fall_amount = FieldType(float, 9)
    stock_rise_and_fall_rate = FieldType(float, 10)
    stock_turnover_rate = FieldType(float, 11)
    stock_volumes = FieldType(float, 12)
    stock_volumes_amount = FieldType(float, 13)
    stock_all_market_cap = FieldType(float, 14)
    stock_circulate_market_cap = FieldType(float, 15)
    def __init__(self, excelPath=None):
        super(stock,self).__init__(excelPath)

class TestDoumentExcel(unittest.TestCase):
    def test_csv(self):
        with stock('C:\\Home\\dev\\Python\\pypi\\src\\2345.csv') as m_result:
            for v in m_result.items():
                #print(v.stock_date)
                self.assertRaises(Exception,m_result)

if __name__ == "__main__":
    unittest.main()


import __init__
import unittest
from TDhelper.Spider.contentExtraction import lineblock
from TDhelper.network.http.http_helper import m_http


class TestSpider(unittest.TestCase):
    def test_spider_contentExtraction(self):
        spider_http = m_http()
        Extraction_set = lineblock(5, 0.2, 50)
        content, statucode = spider_http.getcontent(
            "https://www.sipwps.com/x/113923.html"
        )
        # content= str(content,"utf-8")
        self.assertEqual((statucode,), (200,))
        result = Extraction_set.getBody(content)
        print("正文", result)


if __name__ == "__main__":
    unittest.main()



import __init__
import unittest
from TDhelper.network.http.http_helper import m_http
test_url="http://www.163.com"
H = m_http()

class testHttpHelper(unittest.TestCase):
    def test_post(self):
        result= H.getcontent(test_url)
        print("[INFO]:%s" % result[0])        

if __name__ == "__main__":
    unittest.main()


import io
import os
import sys
import re
import json
def doc(source_str, key):
    result= ''
    isRead= False
    if source_str:
        source_str= io.StringIO(source_str)
        while True:
            m_str= source_str.readline()
            if not m_str:
                break
            m_str= m_str.strip()
            m_str= re.sub(u"\r|\n","",m_str,count=0,flags=0)
            if m_str.lower() == "["+key+"]":
                isRead= True
                m_str= ''
            elif m_str.lower() == "["+key+"end]":
                isRead= False
                break
            if isRead:
                result+=m_str
    if result:
        m_path= result.split('::')
        m_key= None
        if len(m_path)>1:
            m_key= m_path[0:len(m_path)-1]
            result= m_path[len(m_path)-1]
        if result.endswith('.json'):
            m_path= os.path.join(sys.path[0], result)
            m_path= m_path.replace("\\","/")
            with open(m_path,mode='r',encoding='utf-8') as f:
                result= f.read()
                if m_key:
                    result= json.loads(result)
                    for k in m_key:
                        if k in result:
                            result= result[k]
                    if isinstance(result, dict):
                        result= json.dumps(result)
                f.close()
    return result


import __init__
import unittest
from TDhelper.MagicCls.model import model, createCls
from TDhelper.MagicCls.FieldsType import FieldType


class categories(model):
    category_id = FieldType(int, "type_id")
    category_name = FieldType(str, "type_name")

    def __init__(self, source, convert_type="json"):
        super(categories, self).__init__(source, convert_type)


class TestmagicCls(unittest.TestCase):
    def test_magicCls(self):
        api_json = [
            '{"type_id": 0, "type_name": "电影0"}',
            '[{"type_id": 1, "type_name": "电影1"},{"type_id": 2, "type_name": "电影2"},{"type_id": 3, "type_name": "电影3"}]',
            [
                {"type_id": 4, "type_name": "电影4"},
                {"type_id": 5, "type_name": "电影5"},
                {"type_id": 6, "type_name": "电影6"},
            ],
            {"type_id": 7, "type_name": "电影7"},
        ]
        for v in api_json:
            # create cls
            mapping = {"type_id": [int], "name": [str, "type_name"]}
            m_result = createCls("categories", mapping, v)
            self.assertRaises(Exception, m_result)
            print("\r\n-----------------create cls----------------------\r\n")
            for o in m_result.items():
                print(o.type_id)
                print(o.name)

            # mapping object.
            print("\r\n-----------------mapping object----------------------\r\n")
            m_result = categories(v)
            self.assertRaises(Exception, m_result)
            for o in m_result.items():
                print(o.category_id)
                print(o.category_name)


if __name__ == "__main__":
    unittest.main()
